import {
  Button,
  Card,
  Col,
  Popconfirm,
  Row,
  Space,
  Table,
  Typography,
} from "antd";
import { useTranslation } from "react-i18next";
import { useWebSocketContext } from "../../../Contexts/WebSocketContext";
import { SentMessagesCommands } from "../../../Handlers/WebSocketMessageHandler";
import {
  AppStyle,
  Constants,
  FormatDatetime,
  hasPermission,
  myFetch,
  wsConnectionCustomEventName,
} from "../../../utils";
import { useAppContext } from "../../../Contexts/AppContext";
import { useConsolesContext } from "../../../Contexts/ConsolesContext";
import { Link } from "react-router-dom";
import { PlusOutlined } from "@ant-design/icons";
import AddLogManagerServerConnectionModal from "./AddLogManagerServerConnectionModal";
import { useEffect, useState } from "react";

export default function AdminAreaManage() {
  const { t } = useTranslation();
  const appContext = useAppContext();
  const webSocketContext = useWebSocketContext();

  return (
    <>
      {hasPermission(
        appContext.userPermissions,
        Constants.PERMISSIONS.ADMIN_AREA.MANAGE
          .CHECK_WHICH_CATEGORIES_ARE_AVAILABLE
      ) && (
        <Card title={t("adminArea.manage.groupTasksCard.title")}>
          <p>{t("adminArea.manage.groupTasksCard.description")}</p>
          <Button
            onClick={() =>
              webSocketContext.SendSocketMessage(
                SentMessagesCommands.AdminAreaManageCheckWhichCategoriesAreAvailable,
                {}
              )
            }
          >
            {t("adminArea.manage.groupTasksCard.buttonText")}
          </Button>
        </Card>
      )}

      <LogManagerServersTable
        t={t}
        webSocketContext={webSocketContext}
        appContext={appContext}
      />
    </>
  );
}

function LogManagerServersTable({ t, webSocketContext, appContext }) {
  const consolesContext = useConsolesContext();
  const [
    isAddLogManagerServerConnectionModalOpen,
    setIsAddLogManagerServerConnectionModalOpen,
  ] = useState(false);

  const getColumns = () => {
    let columns = [
      {
        title: t("adminArea.manage.logManagers.column.displayName"),
        dataIndex: "displayName",
        key: "displayName",
      },
      {
        title: t("adminArea.manage.logManagers.column.address"),
        dataIndex: "address",
        key: "address",
      },
      {
        title: t("adminArea.manage.logManagers.column.createdAt"),
        dataIndex: "createdAt",
        key: "createdAt",
      },
    ];

    if (
      hasPermission(
        appContext.userPermissions,
        Constants.PERMISSIONS.ADMIN_AREA.MANAGE
          .REMOVE_LOG_MANAGER_SERVER_CONNECTION
      )
    ) {
      columns.push({
        title: t("adminArea.manage.logManagers.column.action"),
        dataIndex: "action",
        key: "action",
        render: (_, record) => (
          <Space size="middle">
            <Popconfirm
              placement="left"
              title={t("adminArea.manage.logManagers.popconfirm.title")}
              description={t(
                "adminArea.manage.logManagers.popconfirm.description"
              )}
              okText={t("common.button.delete")}
              cancelText={t("common.button.cancel")}
              onConfirm={() =>
                webSocketContext.SendSocketMessage(
                  SentMessagesCommands.AdminAreaManageDeleteLogManagerServerConnection,
                  { Id: record.key }
                )
              }
            >
              <Link to="#">{t("common.button.delete")}</Link>
            </Popconfirm>
          </Space>
        ),
      });
    }

    return columns;
  };

  const getTableItems = () => {
    let items = [];

    consolesContext.connectedLogManagerServers.sort(
      (a, b) => new Date(b.CreatedAt) - new Date(a.CreatedAt)
    );

    consolesContext.connectedLogManagerServers.forEach((connection) => {
      items.push({
        key: connection.Id,
        displayName: connection.DisplayName,
        address: connection.Address,
        createdAt: FormatDatetime(connection.CreatedAt),
      });
    });

    return items;
  };

  useEffect(() => {
    const lmscRequest = () =>
      myFetch("/lmsc", "GET").then((data) =>
        consolesContext.setConnectedLogManagerServers(data)
      );

    lmscRequest();

    const handleLmscRequest = () => lmscRequest();

    document.addEventListener(wsConnectionCustomEventName, handleLmscRequest);

    return () =>
      document.removeEventListener(
        wsConnectionCustomEventName,
        handleLmscRequest
      );
  }, []);

  return (
    <>
      <Row style={{ marginTop: AppStyle.app.margin }}>
        <Col xs={24} md={12}>
          <Typography.Title level={4}>
            {t("adminArea.manage.logManagers.header")} (
            {consolesContext.connectedLogManagerServers.length})
          </Typography.Title>
        </Col>

        <Col
          xs={24}
          md={{ span: 10, offset: 2 }}
          xl={{ span: 8, offset: 4 }}
          xxl={{ span: 6, offset: 6 }}
        >
          {hasPermission(
            appContext.userPermissions,
            Constants.PERMISSIONS.ADMIN_AREA.MANAGE
              .ADD_LOG_MANAGER_SERVER_CONNECTION
          ) && (
            <Button
              block
              icon={<PlusOutlined />}
              onClick={() => setIsAddLogManagerServerConnectionModalOpen(true)}
            >
              {t("adminArea.manage.logManagers.addLogManagerServerConnection")}
            </Button>
          )}
        </Col>
      </Row>

      <Table
        scroll={{ x: "max-content" }}
        columns={getColumns()}
        dataSource={getTableItems()}
      />

      <AddLogManagerServerConnectionModal
        isModalOpen={isAddLogManagerServerConnectionModalOpen}
        setIsModalOpen={setIsAddLogManagerServerConnectionModalOpen}
        t={t}
        webSocketContext={webSocketContext}
      />
    </>
  );
}
