import {
  Button,
  Col,
  Collapse,
  Drawer,
  Form,
  Input,
  Row,
  Segmented,
  Select,
  Space,
  Table,
  Tabs,
  Typography,
  notification,
} from "antd";
import { useTranslation } from "react-i18next";
import { useNavigate, useParams } from "react-router-dom";
import {
  AppStyle,
  Constants,
  myFetch,
  wsConnectionCustomEventName,
} from "../../utils";
import { useEffect, useState } from "react";
import { PlusOutlined } from "@ant-design/icons";
import { t } from "i18next";
import PageNotFound from "../PageNotFound";
import { useCrmContext } from "../../Contexts/CrmContext";
import MyPagination from "../../Components/MyPagination";

import "@mdxeditor/editor/style.css";
import { MDXEditor } from "@mdxeditor/editor/MDXEditor";
import { headingsPlugin } from "@mdxeditor/editor/plugins/headings";
import { listsPlugin } from "@mdxeditor/editor/plugins/lists";
import { quotePlugin } from "@mdxeditor/editor/plugins/quote";
import { thematicBreakPlugin } from "@mdxeditor/editor/plugins/thematic-break";
import { toolbarPlugin } from "@mdxeditor/editor/plugins/toolbar";
import { BoldItalicUnderlineToggles } from "@mdxeditor/editor/plugins/toolbar/components/BoldItalicUnderlineToggles";
import { UndoRedo } from "@mdxeditor/editor/plugins/toolbar/components/UndoRedo";
import { linkPlugin } from "@mdxeditor/editor/plugins/link";
import { markdownShortcutPlugin } from "@mdxeditor/editor/plugins/markdown-shortcut";
import { MySupsenseFallback } from "../../Components/MySupsenseFallback";
import { linkDialogPlugin } from "@mdxeditor/editor";
import { tablePlugin } from "@mdxeditor/editor";
import { frontmatterPlugin } from "@mdxeditor/editor";

export default function Crm() {
  const { t } = useTranslation();
  const navigate = useNavigate();
  const crmContext = useCrmContext();
  const { paramType, paramDealPhase } = useParams();
  const [isDrawerOpen, setIsDrawerOpen] = useState(false);
  const [notificationApi, notificationContextHolder] =
    notification.useNotification();

  const title =
    paramType === Constants.CRM_TYPE.CUSTOMERS
      ? "crm.customers.pageTitle"
      : paramType === Constants.CRM_TYPE.DMC_PIPELINE
      ? "crm.dmcPipeline.pageTitle"
      : "crm.setterCloser.pageTitle";

  const segmentedOptions = t(
    paramType === Constants.CRM_TYPE.DMC_PIPELINE
      ? "crm.dmcPipeline.segmentedOptions"
      : "crm.setterCloser.segmentedOptions",
    {
      returnObjects: true,
    }
  );

  const [selectedSegmentedValue, setSelectedSegmentedValue] = useState([]);

  const getTableContent = () => {
    return [
      {
        title: t("crm.table.firstName"),
        dataIndex: "firstName",
        key: "firstName",
      },
      {
        title: t("crm.table.lastName"),
        dataIndex: "lastName",
        key: "lastName",
      },
      {
        title: t("crm.table.createdAt"),
        dataIndex: "createdAt",
        key: "createdAt",
      },
      {
        title: t("crm.table.telephone"),
        dataIndex: "telephone",
        key: "telephone",
      },
      {
        title: t("crm.table.email"),
        dataIndex: "email",
        key: "email",
      },
      {
        title: t("crm.table.lastContact"),
        dataIndex: "lastContact",
        key: "lastContact",
      },
      {
        title: t("crm.table.createdBy"),
        dataIndex: "createdBy",
        key: "createdBy",
      },
    ];
  };

  const getTableItems = () => {
    let items = [];

    crmContext.customers.forEach((item) => {
      items.push({
        key: item.Id,
        id: item.Id,
        firstName: item.FirstName,
        lastName: item.LastName,
        createdAt: item.CreatedAt,
        telephone: item.Telephone,
        email: item.Email,
        lastContact: item.LastContact,
        createdBy: item.CreatedBy,
      });
    });

    return items;
  };

  const onPaginationChange = (page) => {
    crmContext.setPaginationPage(page);
    crmContext.paginationPageRef.current = page;
  };

  useEffect(() => {
    if (paramType === Constants.CRM_TYPE.CUSTOMERS) return;

    setSelectedSegmentedValue(segmentedOptions[paramDealPhase - 1]);
  }, [paramType]);

  useEffect(() => {
    const customersRequest = () =>
      myFetch(
        `/crm/pipeline/${paramType}/${paramDealPhase}?page=${crmContext.paginationPage}`,
        "GET"
      ).then((data) => {
        if (data.Customers !== undefined && data.Customers !== null) {
          crmContext.setCustomers(data.Customers);
        }

        if (data.TotalPages !== undefined) {
          crmContext.setTotalPages(data.TotalPages);
        }
      });

    customersRequest();

    const handleCustomersRequest = () => customersRequest();

    document.addEventListener(
      wsConnectionCustomEventName,
      handleCustomersRequest
    );

    return () =>
      document.removeEventListener(
        wsConnectionCustomEventName,
        handleCustomersRequest
      );
  }, [paramType, paramDealPhase, crmContext.paginationPage]);

  if (paramDealPhase > segmentedOptions.length) {
    return <PageNotFound />;
  }

  return (
    <>
      {notificationContextHolder}

      <div
        style={{
          display: "flex",
          justifyContent: "space-between",
        }}
      >
        <Typography.Title level={4}>{t(title)}</Typography.Title>

        <Button
          icon={<PlusOutlined />}
          onClick={() => {
            crmContext.openDrawerCustomerId.current = "new";
            crmContext.currentDrawerCustomer = null;
            crmContext.currentDrawerCustomerRef.current = null;
            crmContext.changedDrawerCustomerFieldsRef.current = [];

            setIsDrawerOpen(true);
          }}
        >
          {t("crm.buttonNew")}
        </Button>
      </div>

      {paramType !== Constants.CRM_TYPE.CUSTOMERS && (
        <Segmented
          value={selectedSegmentedValue}
          onChange={(value) => {
            setSelectedSegmentedValue(value);
            navigate(
              `/crm/${paramType}/${segmentedOptions.indexOf(value) + 1}`,
              true
            );
          }}
          options={segmentedOptions.map((item) => {
            return {
              value: item,
              label: item,
            };
          })}
          style={{ marginBottom: AppStyle.app.margin }}
        />
      )}

      <Table
        scroll={{ x: "max-content" }}
        columns={getTableContent()}
        dataSource={getTableItems()}
        pagination={false}
        onRow={(record) => {
          return {
            onClick: () => {
              setIsDrawerOpen(true);
              crmContext.openDrawerCustomerId.current = record.id;
            },
          };
        }}
      />

      <MyPagination
        paginationPage={crmContext.paginationPage}
        setPaginationPage={(page) => onPaginationChange(page)}
        totalPages={crmContext.totalPages}
      />

      <CustomerDrawer
        isOpen={isDrawerOpen}
        setIsOpen={setIsDrawerOpen}
        onClose={() => setIsDrawerOpen(false)}
        notificationApi={notificationApi}
      />
    </>
  );
}

function CustomerDrawer({ isOpen, setIsOpen, onClose, notificationApi }) {
  const { t } = useTranslation();
  const crmContext = useCrmContext();

  const tabItems = [
    {
      key: "0",
      label: t("crm.tabs.dealInfo"),
      children: <TabContentDealInfo />,
    },
    {
      key: "1",
      label: t("crm.tabs.activities"),
      children: <TabContentActivities />,
    },
    {
      key: "2",
      label: t("crm.tabs.notes"),
      children: (
        <TabContentNotes
          notes={
            crmContext.currentDrawerCustomer === null ||
            crmContext.currentDrawerCustomer.Notes === null
              ? ""
              : crmContext.currentDrawerCustomer.Notes
          }
        />
      ),
    },
  ];

  const [activeTab, setActiveTab] = useState(tabItems[0].key);

  const customerRequest = () => {
    if (!isOpen) return;

    myFetch(
      `/crm/customer/view/${crmContext.openDrawerCustomerId.current}`,
      "GET"
    ).then((data) => {
      crmContext.setCurrentDrawerCustomer(data);
      crmContext.currentDrawerCustomerRef.current = data;
    });
  };

  const handleRequestError = (status) => {
    if (status === 409) {
      notificationApi["error"]({
        message: t("crm.tabContent.request.duplicateCompanyError.message"),
        description: t(
          "crm.tabContent.request.duplicateCompanyError.description"
        ),
      });
    } else {
      notificationApi["error"]({
        message: "test",
        description: "here",
      });
    }

    setIsOpen(true);
  };

  useEffect(() => {
    if (crmContext.openDrawerCustomerId.current === null) return;

    if (crmContext.openDrawerCustomerId.current === "new") {
      if (isOpen) return;

      // check if something has changed
      if (crmContext.currentDrawerCustomer === null) return;

      myFetch(`/crm/customer/create`, "POST", crmContext.currentDrawerCustomer)
        .then(() => {
          crmContext.openDrawerCustomerId.current = false;
          handleCloseDrawer();
        })
        .catch((status) => handleRequestError(status));
      return;
    }

    if (isOpen) {
      customerRequest();
    } else {
      // check if something has changed
      if (
        JSON.stringify(crmContext.currentDrawerCustomer) !==
        JSON.stringify(crmContext.currentDrawerCustomerRef.current)
      ) {
        // only updated changed fields
        const updatedCustomer = {};

        Object.keys(crmContext.currentDrawerCustomer).forEach((key) => {
          if (
            crmContext.currentDrawerCustomer[key] !==
            crmContext.currentDrawerCustomerRef.current[key]
          ) {
            updatedCustomer[key] = crmContext.currentDrawerCustomer[key];
          }
        });

        myFetch(
          `/crm/customer/update/${crmContext.openDrawerCustomerId.current}`,
          "POST",
          updatedCustomer
        )
          .then(() => handleCloseDrawer())
          .catch((status) => handleRequestError(status));
      }
    }

    const handleCustomerRequest = () => customerRequest();

    document.addEventListener(
      wsConnectionCustomEventName,
      handleCustomerRequest
    );

    return () =>
      document.removeEventListener(
        wsConnectionCustomEventName,
        handleCustomerRequest
      );
  }, [isOpen]);

  const handleCloseDrawer = () => {
    crmContext.openDrawerCustomerId.current = null;
    crmContext.currentDrawerCustomer = null;
    crmContext.currentDrawerCustomerRef.current = null;
    crmContext.changedDrawerCustomerFieldsRef.current = [];

    setActiveTab(tabItems[0].key);
  };

  const title =
    crmContext.currentDrawerCustomer === null
      ? "loading..."
      : `${crmContext.currentDrawerCustomer?.FirstName} ${crmContext.currentDrawerCustomer?.LastName}`;

  return (
    <Drawer
      title={title}
      placement="right"
      open={isOpen}
      onClose={onClose}
      width={720}
      extra={
        <Button
          onClick={() => {
            onClose();
            handleCloseDrawer();
          }}
        >
          {t("crm.buttonUndo")}
        </Button>
      }
    >
      <Tabs
        activeKey={activeTab}
        onChange={(activeKey) => setActiveTab(activeKey)}
        items={tabItems}
        centered
      />
    </Drawer>
  );
}

function Content({ children }) {
  return (
    <Form layout="vertical">
      <Row gutter={16}>
        <Col xs={24} sm={12}>
          {children[0]}
        </Col>
        <Col xs={24} sm={12}>
          {children[1]}
        </Col>
      </Row>
    </Form>
  );
}

function CollapseContainer({ children, label }) {
  return (
    <Collapse
      defaultActiveKey="1"
      items={[
        {
          key: "1",
          label: label,
          children: children,
        },
      ]}
    />
  );
}

function TabContentDealInfo() {
  const crmContext = useCrmContext();

  const addToChangedFields = (value) => {
    if (!crmContext.changedDrawerCustomerFieldsRef.current.includes(value)) {
      crmContext.changedDrawerCustomerFieldsRef.current.push(value);
    }
  };

  return (
    <MySupsenseFallback>
      <Space direction="vertical" style={{ display: "flex" }}>
        <CollapseContainer
          label={t("crm.tabContent.dealInfo.collapseDealStatus.label")}
        >
          <Content>
            <Form.Item
              label={t("crm.tabContent.dealInfo.collapseDealStatus.pipeline")}
              initialValue={crmContext.currentDrawerCustomer?.Pipeline}
            >
              <Select
                value={
                  crmContext.currentDrawerCustomer?.Pipeline === undefined
                    ? 1
                    : crmContext.currentDrawerCustomer?.Pipeline
                }
                onChange={(value) => {
                  crmContext.setCurrentDrawerCustomer({
                    ...crmContext.currentDrawerCustomer,
                    Pipeline: value,
                    DealPhase: 1,
                  });

                  addToChangedFields("Pipeline");
                }}
                options={[
                  {
                    value: 1,
                    label: t("crm.dmcPipeline.pageTitle"),
                  },
                  {
                    value: 2,
                    label: t("crm.setterCloser.pageTitle"),
                  },
                ]}
              />
            </Form.Item>

            <Form.Item
              label={t("crm.tabContent.dealInfo.collapseDealStatus.dealPhase")}
              initialValue={crmContext.currentDrawerCustomer?.DealPhase}
            >
              <Select
                value={
                  crmContext.currentDrawerCustomer?.DealPhase === undefined
                    ? 1
                    : crmContext.currentDrawerCustomer?.DealPhase
                }
                onChange={(value) => {
                  crmContext.setCurrentDrawerCustomer({
                    ...crmContext.currentDrawerCustomer,
                    DealPhase: value,
                  });

                  addToChangedFields("DealPhase");
                }}
                options={t(
                  crmContext.currentDrawerCustomer?.Pipeline === 1
                    ? "crm.dmcPipeline.segmentedOptions"
                    : "crm.setterCloser.segmentedOptions",
                  { returnObjects: true }
                ).map((item, index) => {
                  return {
                    value: index + 1,
                    label: item,
                  };
                })}
              />
            </Form.Item>
          </Content>
        </CollapseContainer>

        <CollapseContainer
          label={t(
            t("crm.tabContent.dealInfo.collapseMasterDataOfContact.label")
          )}
        >
          <Content>
            <Form.Item
              label={t(
                "crm.tabContent.dealInfo.collapseMasterDataOfContact.firstName"
              )}
            >
              <Input
                value={crmContext.currentDrawerCustomer?.FirstName}
                onChange={(e) => {
                  crmContext.setCurrentDrawerCustomer({
                    ...crmContext.currentDrawerCustomer,
                    FirstName: e.target.value,
                  });

                  addToChangedFields("FirstName");
                }}
              />
            </Form.Item>

            <Form.Item
              label={t(
                "crm.tabContent.dealInfo.collapseMasterDataOfContact.lastName"
              )}
            >
              <Input
                value={crmContext.currentDrawerCustomer?.LastName}
                onChange={(e) => {
                  crmContext.setCurrentDrawerCustomer({
                    ...crmContext.currentDrawerCustomer,
                    LastName: e.target.value,
                  });

                  addToChangedFields("LastName");
                }}
              />
            </Form.Item>
          </Content>

          <Content>
            <Form.Item
              label={t(
                "crm.tabContent.dealInfo.collapseMasterDataOfContact.telephone"
              )}
            >
              <Input
                value={crmContext.currentDrawerCustomer?.Telephone}
                onChange={(e) => {
                  crmContext.setCurrentDrawerCustomer({
                    ...crmContext.currentDrawerCustomer,
                    Telephone: e.target.value,
                  });

                  addToChangedFields("Telephone");
                }}
              />
            </Form.Item>

            <Form.Item
              label={t(
                "crm.tabContent.dealInfo.collapseMasterDataOfContact.email"
              )}
            >
              <Input
                value={crmContext.currentDrawerCustomer?.Email}
                onChange={(e) => {
                  crmContext.setCurrentDrawerCustomer({
                    ...crmContext.currentDrawerCustomer,
                    Email: e.target.value,
                  });

                  addToChangedFields("Email");
                }}
              />
            </Form.Item>
          </Content>

          <Content>
            <Form.Item
              label={t(
                "crm.tabContent.dealInfo.collapseMasterDataOfContact.company"
              )}
            >
              <Input
                value={crmContext.currentDrawerCustomer?.Company}
                onChange={(e) => {
                  crmContext.setCurrentDrawerCustomer({
                    ...crmContext.currentDrawerCustomer,
                    Company: e.target.value,
                  });

                  addToChangedFields("Company");
                }}
              />
            </Form.Item>

            <Form.Item
              label={t(
                "crm.tabContent.dealInfo.collapseMasterDataOfContact.zipCode"
              )}
            >
              <Input
                value={crmContext.currentDrawerCustomer?.ZipCode}
                onChange={(e) => {
                  crmContext.setCurrentDrawerCustomer({
                    ...crmContext.currentDrawerCustomer,
                    ZipCode: e.target.value,
                  });

                  addToChangedFields("ZipCode");
                }}
              />
            </Form.Item>
          </Content>

          <Content>
            <Form.Item
              label={t(
                "crm.tabContent.dealInfo.collapseMasterDataOfContact.address"
              )}
            >
              <Input
                value={crmContext.currentDrawerCustomer?.Address}
                onChange={(e) => {
                  crmContext.setCurrentDrawerCustomer({
                    ...crmContext.currentDrawerCustomer,
                    Address: e.target.value,
                  });

                  addToChangedFields("Address");
                }}
              />
            </Form.Item>

            <Form.Item
              label={t(
                "crm.tabContent.dealInfo.collapseMasterDataOfContact.city"
              )}
            >
              <Input
                value={crmContext.currentDrawerCustomer?.City}
                onChange={(e) => {
                  crmContext.setCurrentDrawerCustomer({
                    ...crmContext.currentDrawerCustomer,
                    City: e.target.value,
                  });

                  addToChangedFields("City");
                }}
              />
            </Form.Item>
          </Content>

          <Content>
            <Form.Item
              label={t(
                "crm.tabContent.dealInfo.collapseMasterDataOfContact.country"
              )}
            >
              <Input
                value={crmContext.currentDrawerCustomer?.Country}
                onChange={(e) => {
                  crmContext.setCurrentDrawerCustomer({
                    ...crmContext.currentDrawerCustomer,
                    Country: e.target.value,
                  });

                  addToChangedFields("Country");
                }}
              />
            </Form.Item>

            <Form.Item
              label={t(
                "crm.tabContent.dealInfo.collapseMasterDataOfContact.federalState"
              )}
            >
              <Input
                value={crmContext.currentDrawerCustomer?.FederalState}
                onChange={(e) => {
                  crmContext.setCurrentDrawerCustomer({
                    ...crmContext.currentDrawerCustomer,
                    FederalState: e.target.value,
                  });

                  addToChangedFields("FederalState");
                }}
              />
            </Form.Item>
          </Content>

          <Content>
            <Form.Item
              label={t(
                "crm.tabContent.dealInfo.collapseMasterDataOfContact.website"
              )}
            >
              <Input
                value={crmContext.currentDrawerCustomer?.Website}
                onChange={(e) => {
                  crmContext.setCurrentDrawerCustomer({
                    ...crmContext.currentDrawerCustomer,
                    Website: e.target.value,
                  });

                  addToChangedFields("Website");
                }}
              />
            </Form.Item>

            <Form.Item
              label={t(
                "crm.tabContent.dealInfo.collapseMasterDataOfContact.leadOrigin"
              )}
            >
              <Input
                value={crmContext.currentDrawerCustomer?.LeadOrigin}
                onChange={(e) => {
                  crmContext.setCurrentDrawerCustomer({
                    ...crmContext.currentDrawerCustomer,
                    LeadOrigin: e.target.value,
                  });

                  addToChangedFields("LeadOrigin");
                }}
              />
            </Form.Item>
          </Content>
        </CollapseContainer>

        <CollapseContainer
          label={t(t("crm.tabContent.dealInfo.collapseSetterInfo.label"))}
        >
          <Content>
            <Form.Item
              label={t(
                "crm.tabContent.dealInfo.collapseSetterInfo.numberOfEmployees"
              )}
            >
              <Input
                value={crmContext.currentDrawerCustomer?.NumberOfEmployees}
                onChange={(e) => {
                  crmContext.setCurrentDrawerCustomer({
                    ...crmContext.currentDrawerCustomer,
                    NumberOfEmployees: e.target.value,
                  });

                  addToChangedFields("NumberOfEmployees");
                }}
              />
            </Form.Item>

            <Form.Item
              label={t(
                "crm.tabContent.dealInfo.collapseSetterInfo.numberOfJobsSearchedFor"
              )}
            >
              <Input
                value={
                  crmContext.currentDrawerCustomer?.NumberOfJobsSearchedFor
                }
                onChange={(e) => {
                  crmContext.setCurrentDrawerCustomer({
                    ...crmContext.currentDrawerCustomer,
                    NumberOfJobsSearchedFor: e.target.value,
                  });

                  addToChangedFields("NumberOfJobsSearchedFor");
                }}
              />
            </Form.Item>
          </Content>

          <Content>
            <Form.Item
              label={t("crm.tabContent.dealInfo.collapseSetterInfo.jobTitle")}
            >
              <Input
                value={crmContext.currentDrawerCustomer?.JobTitle}
                onChange={(e) => {
                  crmContext.setCurrentDrawerCustomer({
                    ...crmContext.currentDrawerCustomer,
                    JobTitle: e.target.value,
                  });

                  addToChangedFields("JobTitle");
                }}
              />
            </Form.Item>

            <Form.Item
              label={t(
                "crm.tabContent.dealInfo.collapseSetterInfo.numberOfEmployeesRequired"
              )}
            >
              <Input
                value={
                  crmContext.currentDrawerCustomer?.NumberOfEmployeesRequired
                }
                onChange={(e) => {
                  crmContext.setCurrentDrawerCustomer({
                    ...crmContext.currentDrawerCustomer,
                    NumberOfEmployeesRequired: e.target.value,
                  });

                  addToChangedFields("NumberOfEmployeesRequired");
                }}
              />
            </Form.Item>
          </Content>

          <Content>
            <Form.Item
              label={t(
                "crm.tabContent.dealInfo.collapseSetterInfo.howLongHadHeBeenSearching"
              )}
            >
              <Input
                value={
                  crmContext.currentDrawerCustomer?.HowLongHadHeBeenSearching
                }
                onChange={(e) => {
                  crmContext.setCurrentDrawerCustomer({
                    ...crmContext.currentDrawerCustomer,
                    HowLongHadHeBeenSearching: e.target.value,
                  });

                  addToChangedFields("HowLongHadHeBeenSearching");
                }}
              />
            </Form.Item>

            <Form.Item
              label={t("crm.tabContent.dealInfo.collapseSetterInfo.turnover")}
            >
              <Input
                value={crmContext.currentDrawerCustomer?.Turnover}
                onChange={(e) => {
                  crmContext.setCurrentDrawerCustomer({
                    ...crmContext.currentDrawerCustomer,
                    Turnover: e.target.value,
                  });

                  addToChangedFields("Turnover");
                }}
              />
            </Form.Item>
          </Content>
        </CollapseContainer>

        <CollapseContainer
          label={t(t("crm.tabContent.dealInfo.collapseDealProperties.label"))}
        >
          <Content>
            <Form.Item
              label={t(
                "crm.tabContent.dealInfo.collapseDealProperties.dateOfcompletion"
              )}
            >
              <Input
                value={crmContext.currentDrawerCustomer?.DateOfCompletion}
                onChange={(e) => {
                  crmContext.setCurrentDrawerCustomer({
                    ...crmContext.currentDrawerCustomer,
                    DateOfCompletion: e.target.value,
                  });

                  addToChangedFields("DateOfCompletion");
                }}
              />
            </Form.Item>

            <Form.Item
              label={t(
                "crm.tabContent.dealInfo.collapseDealProperties.orderVolume"
              )}
            >
              <Input
                value={crmContext.currentDrawerCustomer?.OrderVolume}
                onChange={(e) => {
                  crmContext.setCurrentDrawerCustomer({
                    ...crmContext.currentDrawerCustomer,
                    OrderVolume: e.target.value,
                  });

                  addToChangedFields("OrderVolume");
                }}
              />
            </Form.Item>
          </Content>

          <Content>
            <Form.Item
              label={t(
                "crm.tabContent.dealInfo.collapseDealProperties.numberOfInstallments"
              )}
            >
              <Input
                value={crmContext.currentDrawerCustomer?.NumberOfInstallments}
                onChange={(e) => {
                  crmContext.setCurrentDrawerCustomer({
                    ...crmContext.currentDrawerCustomer,
                    NumberOfInstallments: e.target.value,
                  });

                  addToChangedFields("NumberOfInstallments");
                }}
              />
            </Form.Item>

            <Form.Item
              label={t(
                "crm.tabContent.dealInfo.collapseDealProperties.amountsOfTheInstallments"
              )}
            >
              <Input
                value={
                  crmContext.currentDrawerCustomer?.AmountsOfTheInstallments
                }
                onChange={(e) => {
                  crmContext.setCurrentDrawerCustomer({
                    ...crmContext.currentDrawerCustomer,
                    AmountsOfTheInstallments: e.target.value,
                  });

                  addToChangedFields("AmountsOfTheInstallments");
                }}
              />
            </Form.Item>
          </Content>

          <Content>
            <Form.Item
              label={t(
                "crm.tabContent.dealInfo.collapseDealProperties.bookedPackages"
              )}
            >
              <Input
                value={crmContext.currentDrawerCustomer?.BookedPackages}
                onChange={(e) => {
                  crmContext.setCurrentDrawerCustomer({
                    ...crmContext.currentDrawerCustomer,
                    BookedPackages: e.target.value,
                  });

                  addToChangedFields("BookedPackages");
                }}
              />
            </Form.Item>

            <Form.Item
              label={t(
                "crm.tabContent.dealInfo.collapseDealProperties.assignedEmployee"
              )}
            >
              <Input
                value={crmContext.currentDrawerCustomer?.AssignedEmployee}
                onChange={(e) => {
                  crmContext.setCurrentDrawerCustomer({
                    ...crmContext.currentDrawerCustomer,
                    AssignedEmployee: e.target.value,
                  });

                  addToChangedFields("AssignedEmployee");
                }}
              />
            </Form.Item>
          </Content>
        </CollapseContainer>
      </Space>
    </MySupsenseFallback>
  );
}

function TabContentActivities() {
  return (
    <MySupsenseFallback>
      <div>Coming soon</div>
    </MySupsenseFallback>
  );
}

function TabContentNotes({ notes }) {
  const crmContext = useCrmContext();

  return (
    <MySupsenseFallback>
      <MDXEditor
        key={crmContext.currentDrawerCustomer?.Id}
        className="mdx-editor"
        markdown={notes}
        onChange={(value) => {
          crmContext.setCurrentDrawerCustomer({
            ...crmContext.currentDrawerCustomer,
            Notes: value,
          });
        }}
        plugins={[
          listsPlugin(),
          quotePlugin(),
          headingsPlugin(),
          linkPlugin(),
          linkDialogPlugin(),
          tablePlugin(),
          thematicBreakPlugin(),
          frontmatterPlugin(),
          markdownShortcutPlugin(),
          toolbarPlugin({
            toolbarContents: () => (
              <>
                <UndoRedo />
                <BoldItalicUnderlineToggles />
              </>
            ),
          }),
        ]}
      />
    </MySupsenseFallback>
  );
}
