import {
  Button,
  Card,
  Col,
  Popover,
  Result,
  Row,
  Spin,
  Typography,
} from "antd";
import { useEffect, useState } from "react";
import CreateEquipmentDocumentationModal from "./CreateEquipmentDocumentationModal";
import {
  AppStyle,
  Constants,
  FormatDatetime,
  hasPermission,
  myFetch,
} from "../../utils";
import {
  BookOutlined,
  EditOutlined,
  InfoCircleOutlined,
  PlusOutlined,
  ToolOutlined,
} from "@ant-design/icons";
import EditEquipmentDocumentationModal from "./EditEquipmentDocumentationModal";
import { NoteComponent, ScannerComponent } from ".";
import { useTranslation } from "react-i18next";
import { MyAvatar } from "../../Components/MyAvatar";
import { useAppContext } from "../../Contexts/AppContext";
import { useNavigate, useParams } from "react-router-dom";
import MyPagination from "../../Components/MyPagination";

export default function ViewEquipmentDocumentations({
  isEditEquipmentDocumentationModalOpen,
  isCreateEquipmentDocumentationModalOpen,
}) {
  const { t } = useTranslation();
  const navigate = useNavigate();
  const appContext = useAppContext();
  const { paramStockId, paramDocumentationId } = useParams();

  const [equipmentDocumentationResponse, setEquipmentDocumentationResponse] =
    useState(null);
  const [isEquipmentDocumentationLoading, setIsEquipmentDocumentationLoading] =
    useState(false);
  const [paginationPage, setPaginationPage] = useState(1);

  const fetchDocumentation = (page = 1) => {
    setIsEquipmentDocumentationLoading(true);

    myFetch(
      `/equipment/documentations/${paramStockId}?page=${page}`,
      "GET"
    ).then((data) => {
      const updatedData = { ...data };

      // sort by date
      // last created will be on top of the list
      updatedData.Documentations = data.Documentations.sort(
        (a, b) => new Date(b.CreatedAt) - new Date(a.CreatedAt)
      );

      setIsEquipmentDocumentationLoading(false);
      setEquipmentDocumentationResponse(updatedData);
    });
  };

  useEffect(() => {
    if (paginationPage === 1) {
      fetchDocumentation(1);
    } else {
      setPaginationPage(1);
    }
  }, [paramStockId]);

  // triggers on pagination change
  useEffect(() => fetchDocumentation(paginationPage), [paginationPage]);

  const CreateEquipmentDocumentationButton = ({ buttonBlock }) => {
    if (
      !hasPermission(
        appContext.userPermissions,
        Constants.PERMISSIONS.EQUIPMENT_DOCUMENTATION.CREATE
      )
    )
      return null;

    return (
      <Button
        block={buttonBlock}
        type="primary"
        icon={<PlusOutlined />}
        onClick={() => navigate(`create`)}
      >
        {t("createEquipmentDocumentationModal.buttonCreateDocumentation")}
      </Button>
    );
  };

  const InvexStockItemThumbnail = ({ width }) => {
    return (
      <img
        src={`${Constants.API_ADDRESS}/equipment/thumbnail/${paramStockId}`}
        width={width}
        style={{ borderRadius: 6 }}
      />
    );
  };

  const DocumentationContent = ({ documentation }) => {
    return (
      <Card style={{ marginTop: AppStyle.app.margin }}>
        <div
          style={{
            display: "flex",
            flexDirection: "row",
            gap: 10,
            justifyContent: "space-between",
            marginBottom: AppStyle.typography.text.marginBottom,
          }}
        >
          <div
            style={{
              display: "flex",
              flexDirection: "row",
              gap: 10,
              alignItems: "center",
            }}
          >
            <DocumentationTypeIcon type={documentation.Type} />
            <Typography.Title level={4} style={{ margin: 0 }}>
              {documentation.Title}
            </Typography.Title>
          </div>

          <div
            style={{
              display: "flex",
              flexDirection: "row",
              gap: 10,
              alignItems: "center",
            }}
          >
            <MyAvatar
              allUsers={appContext.users}
              userId={documentation.CreatedByUserId}
              tooltip
            />

            <Popover
              title={
                <Typography.Title
                  level={4}
                  style={{ color: Constants.COLORS.SECONDARY }}
                >
                  {t("viewEquipmentDocumentations.detailsPopover.title")}
                </Typography.Title>
              }
              trigger="click"
              placement="left"
              content={
                <p style={{ color: "#000", margin: 0 }}>
                  <b>{t("common.text.id")}</b> {documentation.Id}
                  <br />
                  <b>{t("common.text.type")}</b>{" "}
                  {
                    t(
                      "equipmentDocumentationViewEditComponent.selectDocumentationTypeOptions",
                      { returnObjects: true }
                    )[documentation.Type - 1]?.label
                  }
                  <br />
                  <b>{t("common.text.createdAt")}</b>{" "}
                  {FormatDatetime(documentation.CreatedAt)}
                  <br />
                  <b>{t("common.text.updatedAt")}</b>{" "}
                  {FormatDatetime(documentation.UpdatedAt)}
                </p>
              }
            >
              <InfoCircleOutlined
                style={{ fontSize: 24, color: Constants.COLORS.ICON_INFO }}
              />
            </Popover>

            {hasPermission(
              appContext.userPermissions,
              Constants.PERMISSIONS.EQUIPMENT_DOCUMENTATION.EDIT
            ) && (
              <EditOutlined
                style={{ fontSize: 24 }}
                onClick={() => navigate(`edit/${documentation.Id}`)}
              />
            )}
          </div>
        </div>

        {documentation.Notes !== "" &&
          JSON.parse(documentation.Notes).map((note, index) => (
            <NoteComponent
              key={index}
              viewMode
              image={note.Image}
              documentationId={documentation.Id}
              description={note.Description}
            />
          ))}
      </Card>
    );
  };

  const CreateEquipmentDocumentationModalComponent = () => {
    return (
      <CreateEquipmentDocumentationModal
        isOpen={isCreateEquipmentDocumentationModalOpen}
        stockItemId={paramStockId}
        fetchDocumentation={fetchDocumentation}
        buttonBlock={true}
      />
    );
  };

  if (isEquipmentDocumentationLoading) {
    return (
      <>
        <ScannerComponent />

        <div
          style={{
            display: "flex",
            alignItems: "center",
            justifyContent: "center",
            marginTop: 120,
          }}
        >
          <Spin size="large" />
        </div>
      </>
    );
  }

  if (equipmentDocumentationResponse === null) return <ScannerComponent />;

  // backend unauthorized to access invex
  if (equipmentDocumentationResponse.Status === 401) {
    return (
      <>
        <ScannerComponent />

        <Result
          status="403"
          title={t("viewEquipmentDocumentations.result403.title")}
          subTitle={t("viewEquipmentDocumentations.result403.description")}
        />
      </>
    );
  }

  // stock item doesn't exists on invex
  if (equipmentDocumentationResponse.Status === 404) {
    return (
      <>
        <ScannerComponent />

        <Result
          status="500"
          title={t("viewEquipmentDocumentations.result500.title")}
          subTitle={t("viewEquipmentDocumentations.result500.description")}
        />
      </>
    );
  }

  if (equipmentDocumentationResponse.Documentations.length === 0) {
    return (
      <>
        <ScannerComponent />

        <Result
          status="404"
          title={t("viewEquipmentDocumentations.result404.title")}
          subTitle={t("viewEquipmentDocumentations.result404.description")}
          extra={[
            <CreateEquipmentDocumentationButton key="0" buttonBlock={false} />,
          ]}
        />

        <CreateEquipmentDocumentationModalComponent />
      </>
    );
  }

  return (
    <>
      <ScannerComponent />

      <Row gutter={AppStyle.grid.row.gutter} style={{ alignItems: "center" }}>
        <Col
          xs={24}
          sm={{ span: 8 }}
          md={{ span: 6 }}
          style={{
            display: "flex",
            flexDirection: "row",
            gap: 10,
            alignItems: "center",
          }}
        >
          {equipmentDocumentationResponse.Documentations.length !== 0 && (
            <Popover
              placement="right"
              trigger="hover"
              content={<InvexStockItemThumbnail width={256} />}
            >
              <>
                <InvexStockItemThumbnail width={64} />
              </>
            </Popover>
          )}

          <Typography.Title level={4} style={{ margin: 0 }}>
            {paramStockId}
          </Typography.Title>
        </Col>

        <Col xs={24} sm={{ span: 8, offset: 8 }} md={{ span: 6, offset: 12 }}>
          <CreateEquipmentDocumentationButton buttonBlock={true} />
        </Col>
      </Row>

      {equipmentDocumentationResponse.Documentations.map(
        (documentation, index) => (
          <DocumentationContent key={index} documentation={documentation} />
        )
      )}

      <MyPagination
        paginationPage={paginationPage}
        setPaginationPage={(page) => setPaginationPage(page)}
        totalPages={equipmentDocumentationResponse.TotalPages}
      />

      <EditEquipmentDocumentationModal
        isOpen={isEditEquipmentDocumentationModalOpen}
        stockItemId={paramStockId}
        documentationId={paramDocumentationId}
        fetchDocumentation={fetchDocumentation}
      />

      <CreateEquipmentDocumentationModalComponent />
    </>
  );
}

function DocumentationTypeIcon({ type }) {
  switch (type) {
    case 1:
      return <ToolOutlined style={{ fontSize: 24 }} />;
    case 2:
      return <BookOutlined style={{ fontSize: 24 }} />;
    default:
      console.error("DocumentationTypeIcon type not found:", type);
      return null;
  }
}
