import {
  Alert,
  Button,
  Checkbox,
  Form,
  Input,
  InputNumber,
  Popover,
  Space,
  Steps,
  Typography,
  notification,
} from "antd";
import { useEffect, useRef } from "react";
import { useNavigate, useParams } from "react-router-dom";
import {
  AppStyle,
  BrowserTabSession,
  Constants,
  FormatDatetime,
  GetDuration,
  hasXYPermission,
  myFetch,
  wsConnectionCustomEventName,
} from "../../../utils";
import {
  CheckOutlined,
  InfoCircleOutlined,
  LockOutlined,
  RetweetOutlined,
  UndoOutlined,
} from "@ant-design/icons";
import TextArea from "antd/es/input/TextArea";
import { useTranslation } from "react-i18next";
import { MyAvatar } from "../../../Components/MyAvatar";
import MyModal, { MyNotFoundModalContent } from "../../../Components/MyModal";
import MyAttachments from "../../../Components/MyAttachments";
import { useWebSocketContext } from "../../../Contexts/WebSocketContext";
import { useGroupTasksContext } from "../../../Contexts/GroupTasksContext";
import { useAppContext } from "../../../Contexts/AppContext";
import { SentMessagesCommands } from "../../../Handlers/WebSocketMessageHandler";
import {
  SelectComponent,
  SelectMachineComponent,
} from "./GroupTypeSelectionModal";
import { MyFormItem, MyFormItemLabel } from ".";

export default function GroupTasksViewModal({ isOpen }) {
  const webSocketContext = useWebSocketContext();
  const appContext = useAppContext();
  const groupTasksContext = useGroupTasksContext();
  const navigate = useNavigate();
  const [notificationApi, notificationContextHolder] =
    notification.useNotification();
  let { paramCategory, paramGroupTaskId } = useParams();
  const { t } = useTranslation();
  const currentGroupTask = useRef(null);

  const handleCancel = () => {
    currentGroupTask.current = null;
    navigate(`${Constants.ROUTE_PATHS.GROUP_TASKS}${paramCategory}`);
  };

  useEffect(() => {
    if (!isOpen) return;

    const stepsRequest = () =>
      myFetch(
        `/grouptasks/${paramCategory}/steps/${paramGroupTaskId}`,
        "GET"
      ).then((data) => {
        currentGroupTask.current = data.GroupTask;
        groupTasksContext.setGroupTasksSteps(data.GroupTaskSteps);

        // set the select inputs

        if (
          data.GroupTaskSteps[data.GroupTaskSteps.length - 1].Inputs !== "" &&
          data.GroupTaskSteps[data.GroupTaskSteps.length - 1].Status ===
            Constants.GROUP_TASKS_STATUS.INPUT_REQUIRED
        ) {
          const inputs = JSON.parse(
            data.GroupTaskSteps[data.GroupTaskSteps.length - 1].Inputs
          );

          groupTasksContext.setSelectInputs(inputs);
        }
      });

    stepsRequest();

    const handleWsConnectionEvent = () => stepsRequest();

    document.addEventListener(
      wsConnectionCustomEventName,
      handleWsConnectionEvent
    );

    return () =>
      document.removeEventListener(
        wsConnectionCustomEventName,
        handleWsConnectionEvent
      );
  }, [isOpen, paramCategory]);

  if (!isOpen) return <></>;

  const getAlertType = (status) => {
    switch (status) {
      case Constants.GROUP_TASKS_STATUS.FINISHED:
        return "success";
      case Constants.GROUP_TASKS_STATUS.FAILED:
        return "error";
      case Constants.GROUP_TASKS_STATUS.CANCELED:
      case Constants.GROUP_TASKS_STATUS.PAUSED:
      case Constants.GROUP_TASKS_STATUS.UNDO_ENDED:
        return "warning";
      case Constants.GROUP_TASKS_STATUS.INPUT_REQUIRED:
      case Constants.GROUP_TASKS_STATUS.RUNNING:
      default:
        return "info";
    }
  };

  const getAlertMessage = (status) => {
    switch (status) {
      case Constants.GROUP_TASKS_STATUS.FINISHED:
        return t("groupTasks.groupTasksViewModal.alertMessage.successful");
      case Constants.GROUP_TASKS_STATUS.RUNNING:
        return t("groupTasks.groupTasksViewModal.alertMessage.taskIsRunning");
      case Constants.GROUP_TASKS_STATUS.CANCELED:
        return t("groupTasks.groupTasksViewModal.alertMessage.taskCanceled");
      case Constants.GROUP_TASKS_STATUS.FAILED:
        return t("groupTasks.groupTasksViewModal.alertMessage.taskFailed");
      case Constants.GROUP_TASKS_STATUS.INPUT_REQUIRED:
        return t(
          "groupTasks.groupTasksViewModal.alertMessage.taskInputRequired"
        );
      case Constants.GROUP_TASKS_STATUS.PAUSED:
        return t("groupTasks.groupTasksViewModal.alertMessage.paused");
      case Constants.GROUP_TASKS_STATUS.UNDO_ENDED:
        return t("groupTasks.groupTasksViewModal.alertMessage.undoEnded");
      default:
        return "Alert message not found";
    }
  };

  const getStepItemStatus = (status) => {
    switch (status) {
      case Constants.GROUP_TASKS_STATUS.FINISHED:
        return "finish";
      case Constants.GROUP_TASKS_STATUS.RUNNING:
        return "process";
      case Constants.GROUP_TASKS_STATUS.CANCELED:
      case Constants.GROUP_TASKS_STATUS.FAILED:
        return "error";
      case Constants.GROUP_TASKS_STATUS.INPUT_REQUIRED:
      case Constants.GROUP_TASKS_STATUS.PAUSED:
      case Constants.GROUP_TASKS_STATUS.UNDO_ENDED:
      default:
        return "wait";
    }
  };

  const handleTaskFailedTryAgainRunTaskStep = (taskStepId, step) => {
    webSocketContext.SendSocketMessage(
      SentMessagesCommands.TaskFailedTryAgainRunTaskStep,
      {
        groupTaskId: currentGroupTask.current.Id,
        category: currentGroupTask.current.Category,
        groupId: currentGroupTask.current.GroupId,
        step: step,
        taskStepId: taskStepId,
      }
    );
  };

  const handleTaskContinueTaskStep = (taskStepId, step) => {
    const groupTasksViewModalRequiredInputsForm = document.getElementById(
      "groupTasksViewModalRequiredInputsForm"
    );

    let canTaskContinued = true;
    let taskInputs = {}; // example format: { "parameterNameAsKey": { value: "value", data: "data" } }
    let foundSelectInputs = [];

    if (groupTasksViewModalRequiredInputsForm !== null) {
      const specifiedTaskInputs =
        groupTasksViewModalRequiredInputsForm.getElementsByTagName("input");

      if (specifiedTaskInputs.length > 0) {
        for (let i = 0; i < specifiedTaskInputs.length; i++) {
          if (specifiedTaskInputs[i].type === "search") {
            foundSelectInputs.push(specifiedTaskInputs[i].id.split("-")[6]);
            continue;
          }

          if (specifiedTaskInputs[i].type === "checkbox") {
            taskInputs[specifiedTaskInputs[i].id.split("-")[6]] = {
              value: specifiedTaskInputs[i].checked.toString(),
            };
            continue;
          }

          if (specifiedTaskInputs[i].value === "") {
            canTaskContinued = false;
            break;
          }

          taskInputs[specifiedTaskInputs[i].id.split("-")[6]] = {
            value: specifiedTaskInputs[i].value,
          };
        }
      }

      const specifiedTaskTextareas =
        groupTasksViewModalRequiredInputsForm.getElementsByTagName("textarea");

      if (specifiedTaskTextareas.length > 0) {
        for (let i = 0; i < specifiedTaskTextareas.length; i++) {
          if (specifiedTaskTextareas[i].value === "") {
            canTaskContinued = false;
            break;
          }

          taskInputs[specifiedTaskTextareas[i].id.split("-")[6]] = {
            value: specifiedTaskTextareas[i].value,
          };
        }
      }
    }

    const selectInputsObjectKeys = Object.keys(groupTasksContext.selectInputs);

    // if there are select inputs, we need to check if they are all selected
    if (foundSelectInputs.length > 0) {
      for (let i = 0; i < foundSelectInputs.length; i++) {
        if (!selectInputsObjectKeys.includes(foundSelectInputs[i])) {
          canTaskContinued = false;
          break;
        } else {
          taskInputs[foundSelectInputs[i]] = {
            value: groupTasksContext.selectInputs[foundSelectInputs[i]].value,
          };
        }
      }
    }

    if (!canTaskContinued) {
      notificationApi["error"]({
        message: t(
          "groupTasks.groupTasksViewModal.notification.inputsCannotBeEmpty.message"
        ),
        description: t(
          "groupTasks.groupTasksViewModal.notification.inputsCannotBeEmpty.description"
        ),
      });
      return;
    }

    webSocketContext.SendSocketMessage(
      SentMessagesCommands.TaskContinueTaskStep,
      {
        groupTaskId: currentGroupTask.current.Id,
        category: currentGroupTask.current.Category,
        groupId: currentGroupTask.current.GroupId,
        step: step,
        taskStepId: taskStepId,
        taskInputs: taskInputs,
      }
    );

    // reset the select inputs
    groupTasksContext.setSelectInputs({});
  };

  const handleUserActionTaskStep = (action, taskStepId, step) => {
    webSocketContext.SendSocketMessage(
      SentMessagesCommands.HandleUserActionTaskStep,
      {
        action: action,
        groupTaskId: currentGroupTask.current.Id,
        category: currentGroupTask.current.Category,
        groupId: currentGroupTask.current.GroupId,
        step: step,
        taskStepId: taskStepId,
      }
    );
  };

  const ActionHandler = ({ status, taskStepId, index, taskLocked }) => {
    const currentStepTask = groupTasksContext.categoryGroup.groups.find(
      (group) => group.id === currentGroupTask.current.GroupId
    ).tasks[index];

    switch (status) {
      case Constants.GROUP_TASKS_STATUS.FAILED:
        return (
          <Button
            size="small"
            danger
            disabled={taskLocked}
            onClick={() =>
              handleTaskFailedTryAgainRunTaskStep(taskStepId, index + 1)
            }
          >
            {t("common.button.tryAgain")}
          </Button>
        );
      case Constants.GROUP_TASKS_STATUS.INPUT_REQUIRED:
        return (
          <Button
            size="small"
            disabled={taskLocked}
            onClick={() => handleTaskContinueTaskStep(taskStepId, index + 1)}
          >
            {t("common.button.continue")}
          </Button>
        );
      case Constants.GROUP_TASKS_STATUS.PAUSED:
        return (
          <Space>
            {currentStepTask.repeatPossible && (
              <Button
                icon={<RetweetOutlined />}
                size="small"
                disabled={taskLocked}
                onClick={() =>
                  handleUserActionTaskStep(2, taskStepId, index + 1)
                }
              >
                {t("common.button.repeat")}
              </Button>
            )}

            {currentStepTask.undoPossible && (
              <Button
                icon={<UndoOutlined />}
                size="small"
                disabled={taskLocked}
                onClick={() =>
                  handleUserActionTaskStep(1, taskStepId, index + 1)
                }
              >
                {t("common.button.undo")}
              </Button>
            )}

            <Button
              icon={<CheckOutlined />}
              size="small"
              type={
                currentStepTask.repeatPossible && currentStepTask.undoPossible
                  ? "primary"
                  : "default"
              }
              disabled={taskLocked}
              onClick={() => handleUserActionTaskStep(0, taskStepId, index + 1)}
            >
              {t("common.button.resume")}
            </Button>
          </Space>
        );
      case Constants.GROUP_TASKS_STATUS.UNDO_ENDED:
        return (
          <Button
            size="small"
            disabled={taskLocked}
            onClick={() => handleUserActionTaskStep(2, taskStepId, index + 1)}
          >
            {t("common.button.resume")}
          </Button>
        );

      default:
        return <></>;
    }
  };

  const stepsItemHandler = () => {
    let groupTaskSteps = [];
    let groupTasks = [];

    groupTasksContext.groupTasksSteps.forEach((step) => {
      if (step.GroupTasksId === paramGroupTaskId) {
        groupTaskSteps.push(step);
      }
    });

    groupTaskSteps.sort((a, b) => a.Step - b.Step);

    if (groupTasksContext.categoryGroup.groups === undefined) return [];

    groupTasksContext.categoryGroup.groups.forEach((group) => {
      if (currentGroupTask.current.GroupId === group.id) {
        groupTasks = group.tasks;
        return;
      }
    });

    const getStepItem = (groupTask, index) => {
      return {
        key: index,
        title:
          groupTaskSteps[index] !== undefined &&
          groupTaskSteps[index].Inputs !== "" &&
          groupTaskSteps[index].Status !==
            Constants.GROUP_TASKS_STATUS.INPUT_REQUIRED &&
          groupTaskSteps[index].Status !==
            Constants.GROUP_TASKS_STATUS.UNDO_ENDED ? (
            <>
              {groupTask.name}{" "}
              <Popover
                key={groupTaskSteps[index].Id + index}
                trigger="click"
                placement="bottomLeft"
                title={
                  <h2
                    style={{
                      fontWeight: "bold",
                      color: Constants.COLORS.SECONDARY,
                    }}
                  >
                    {t(
                      "groupTasks.groupTasksViewModal.popover.specifiedTaskInputs"
                    )}
                  </h2>
                }
                content={
                  <>
                    {groupTask.parameters.length > 0 &&
                      groupTask.parameters.map((task) => {
                        return (
                          <span key={task.parameterName}>
                            <span style={{ fontWeight: "bold" }}>
                              {task.displayName}:{" "}
                            </span>
                            <span>
                              {
                                JSON.parse(groupTaskSteps[index].Inputs)[
                                  task.parameterName
                                ]?.value
                              }
                            </span>
                            <br />
                          </span>
                        );
                      })}
                  </>
                }
              >
                <InfoCircleOutlined
                  style={{
                    fontSize: "18px",
                    color: Constants.COLORS.ICON_INFO,
                    marginBottom: "4px",
                    verticalAlign: "middle",
                  }}
                />
              </Popover>{" "}
              {groupTaskSteps[index]?.CreatorUserId !== undefined && (
                <MyAvatar
                  tooltip
                  allUsers={appContext.users}
                  userId={groupTaskSteps[index].CreatorUserId}
                />
              )}
            </>
          ) : (
            <>
              {groupTask.name}{" "}
              {groupTaskSteps[index]?.CreatorUserId !== undefined && (
                <MyAvatar
                  tooltip
                  allUsers={appContext.users}
                  userId={groupTaskSteps[index].CreatorUserId}
                />
              )}
            </>
          ),
        status:
          groupTaskSteps[index] !== undefined
            ? getStepItemStatus(groupTaskSteps[index].Status)
            : "wait",
        description:
          groupTaskSteps[index] !== undefined ? (
            <>
              <p style={{ color: "#000" }}>
                <b>ID:</b> {groupTaskSteps[index].Id}
                <br />
                <b>{t("groupTasks.groupTasksViewModal.startedAt")}:</b>{" "}
                {FormatDatetime(groupTaskSteps[index].StartedAt)}
                <br />
                <b>{t("groupTasks.groupTasksViewModal.endedAt")}:</b>{" "}
                {groupTaskSteps[index].EndedAt !== "0001-01-01T00:00:00Z"
                  ? FormatDatetime(groupTaskSteps[index].EndedAt)
                  : Constants.TEXT_EMPTY_PLACEHOLDER}
                <br />
                <b>{t("groupTasks.groupTasksViewModal.duration")}:</b>{" "}
                {GetDuration(
                  groupTaskSteps[index].StartedAt,
                  groupTaskSteps[index].EndedAt
                )}
              </p>

              <Alert
                style={{ width: "100%" }}
                message={
                  <div
                    style={{
                      display: "flex",
                      justifyContent: "space-between",
                    }}
                  >
                    <b>
                      {getAlertMessage(groupTaskSteps[index].Status)}{" "}
                      {groupTaskSteps[index].LockedByUserId !== "" && (
                        <>
                          <LockOutlined />{" "}
                          <MyAvatar
                            tooltip
                            allUsers={appContext.users}
                            userId={groupTaskSteps[index].LockedByUserId}
                            avatarWidth={24}
                          />
                        </>
                      )}
                    </b>
                    <ActionHandler
                      status={groupTaskSteps[index].Status}
                      index={index}
                      taskLocked={groupTaskSteps[index].LockedByUserId}
                      taskStepId={groupTaskSteps[index].Id}
                    />
                  </div>
                }
                description={
                  groupTaskSteps[index].Status ===
                  Constants.GROUP_TASKS_STATUS.INPUT_REQUIRED ? (
                    <div id={`${currentGroupTask.current.Id}-scroll-${index}`}>
                      <InputRequiredHandler
                        webSocketContext={webSocketContext}
                        appContext={appContext}
                        currentGroupTask={currentGroupTask.current}
                        groupTaskParameters={groupTask.parameters}
                        groupTaskStepInputs={
                          groupTaskSteps[index] !== undefined &&
                          groupTaskSteps[index].Inputs !== "" &&
                          groupTaskSteps[index].Inputs
                        }
                        notificationApi={notificationApi}
                        step={index + 1}
                        taskLockedByUserId={
                          groupTaskSteps[index].LockedByUserId
                        }
                      />
                    </div>
                  ) : (
                    groupTaskSteps[index].Log.length > 0 && (
                      <div
                        id={`${currentGroupTask.current.Id}-scroll-${index}`}
                        style={{ width: "100%" }}
                      >
                        <GroupTaskStepLogHandler
                          currentGroupTaskId={currentGroupTask.current.Id}
                          log={groupTaskSteps[index].Log}
                          files={groupTaskSteps[index].Files}
                        />
                      </div>
                    )
                  )
                }
                type={getAlertType(groupTaskSteps[index].Status)}
                showIcon
              />
            </>
          ) : (
            ""
          ),
      };
    };

    let stepItems = [];

    groupTasks.forEach((groupTask, index) =>
      stepItems.push(getStepItem(groupTask, index))
    );

    // occurs when tasks were taken from the group task config, but at a previous time the tasks existed
    if (currentGroupTask.current.NumberOfSteps > groupTasks.length) {
      for (
        let i = groupTasks.length;
        i < currentGroupTask.current.NumberOfSteps;
        i++
      ) {
        let stepParams = [];

        if (groupTaskSteps[i].Inputs !== "") {
          let params = JSON.parse(groupTaskSteps[i].Inputs);

          for (let i2 = 0; i2 < params.length; i2++) {
            params[i2].displayName = params[i2].parameterName;
          }

          stepParams = params;
        }

        stepItems.push(getStepItem({ name: "???", parameters: stepParams }, i));
      }
    }

    return stepItems;
  };

  const GlobalInputs = () => {
    if (
      currentGroupTask.current.GlobalInputs === null ||
      currentGroupTask.current.GlobalInputs === "[]"
    )
      return <></>;

    const parsedGlobalInputs = JSON.parse(
      currentGroupTask.current.GlobalInputs
    );

    if (parsedGlobalInputs === null) return <></>;

    return (
      <>
        <h2
          style={{
            fontWeight: "bold",
            color: Constants.COLORS.SECONDARY,
          }}
        >
          {t("groupTasks.groupTasksViewModal.popover.specifiedGlobalInputs")}
        </h2>

        <span>
          {Object.keys(parsedGlobalInputs).map((globalInput) => (
            <span key={globalInput}>
              <span style={{ fontWeight: "bold" }}>
                {
                  groupTasksContext.categoryGroup.groups
                    .find(
                      (group) => group.id === currentGroupTask.current.GroupId
                    )
                    .globalInputs.find(
                      (gI) =>
                        gI.parameterName ===
                        parsedGlobalInputs[globalInput].parameterName
                    ).displayName
                }
                :{" "}
              </span>
              <span>{parsedGlobalInputs[globalInput].value}</span>
              <br />
            </span>
          ))}
        </span>
      </>
    );
  };

  const PopoverContent = () => {
    // get current group task from useState as endetAt and duration not updated only with currentGroupTask as it is a useRef
    const cGroupTask = groupTasksContext.groupTasks.find(
      (g) => g.Id === currentGroupTask.current.Id
    );

    return (
      <div key={paramGroupTaskId}>
        <h1
          style={{
            color: Constants.COLORS.PRIMARY,
            fontWeight: "bold",
            marginBottom: 0,
          }}
        >
          {currentGroupTask.current.GroupName}{" "}
          <Popover
            trigger="click"
            title={
              <h2
                style={{
                  fontWeight: "bold",
                  color: Constants.COLORS.SECONDARY,
                }}
              >
                {t("groupTasks.groupTasksViewModal.popover.details")}
              </h2>
            }
            content={
              <>
                <p style={{ margin: 0 }}>
                  <span style={{ fontWeight: "bold" }}>ID:</span>{" "}
                  {paramGroupTaskId} <br />
                  <span style={{ fontWeight: "bold" }}>
                    {t("groupTasks.groupTasksViewModal.category")}:
                  </span>{" "}
                  {currentGroupTask.current.Category}
                  <br />
                  <span style={{ fontWeight: "bold" }}>
                    {t("groupTasks.groupTasksViewModal.startedAt")}:
                  </span>{" "}
                  {FormatDatetime(currentGroupTask.current.StartedAt)}
                  <br />
                  <span style={{ fontWeight: "bold" }}>
                    {t("groupTasks.groupTasksViewModal.endedAt")}:
                  </span>{" "}
                  {FormatDatetime(cGroupTask?.EndedAt)}
                  <br />
                  <span style={{ fontWeight: "bold" }}>
                    {t("groupTasks.groupTasksViewModal.duration")}:
                  </span>{" "}
                  {GetDuration(cGroupTask?.StartedAt, cGroupTask?.EndedAt)}
                </p>

                <GlobalInputs />
              </>
            }
          >
            <InfoCircleOutlined
              style={{
                fontSize: "18px",
                color: Constants.COLORS.ICON_INFO,
                marginBottom: "4px",
                verticalAlign: "middle",
              }}
            />
          </Popover>{" "}
          <MyAvatar
            tooltip
            allUsers={appContext.users}
            userId={currentGroupTask.current.CreatorUserId}
          />
          <br />
        </h1>
        <div style={{ marginBottom: 16 }}>
          <span>{currentGroupTask.current.Description}</span>
        </div>

        <Steps direction="vertical" items={stepsItemHandler()} />
      </div>
    );
  };

  return (
    <MyModal isOpen={isOpen} onCancel={handleCancel}>
      {notificationContextHolder}

      {!currentGroupTask.current ||
      groupTasksContext === null ||
      !hasXYPermission(
        appContext.userPermissions,
        Constants.PERMISSIONS.GROUP_TASKS.OVERVIEW.XYView,
        currentGroupTask.current.Category
      ) ? (
        <MyNotFoundModalContent
          resultTitle={t("groupTasks.groupTasksViewModal.groupTaskNotFound")}
        />
      ) : (
        <PopoverContent />
      )}
    </MyModal>
  );
}

function InputRequiredHandler({
  webSocketContext,
  appContext,
  currentGroupTask,
  groupTaskParameters,
  groupTaskStepInputs,
  step,
  taskLockedByUserId,
  notificationApi,
}) {
  const { t } = useTranslation();
  const groupTasksContext = useGroupTasksContext();

  const globalInputs = JSON.parse(currentGroupTask.GlobalInputs);
  const stepInputs = JSON.parse(groupTaskStepInputs);

  const getDefaultValue = (groupTaskParameter) => {
    if (stepInputs !== false) {
      const stepInput = stepInputs[groupTaskParameter.parameterName]?.value;

      if (stepInput !== undefined && stepInput !== null) {
        return stepInput;
      }
    }

    if (globalInputs === undefined || !groupTaskParameter.global) return "";

    const globalInputValue = globalInputs.find(
      (globalInput) =>
        globalInput.parameterName === groupTaskParameter.parameterName
    );

    return globalInputValue !== undefined ? globalInputValue.value : "";
  };

  let lastChange = useRef();
  let typingTimer = useRef();

  const sendTypingMessage = (
    inputType,
    currentGroupTaskId,
    groupTaskParameterName,
    inputValue,
    notes
  ) => {
    const data = {
      category: currentGroupTask.Category, // needed for ws topic
      lockedByUserId: appContext.userId.current,
      groupTaskId: currentGroupTaskId,
      parameterName: groupTaskParameterName,
      value: inputValue,
      step: step,
      rememberId: BrowserTabSession,
      inputType: inputType,
    };

    if (inputType === "text") {
      // element is only needed for text and checkbox inputs for the DOM manipulation
      data.element = `${currentGroupTaskId}-${step}-${groupTaskParameterName}`;
    } else {
      data.data = notes;
    }

    webSocketContext.SendSocketMessage(SentMessagesCommands.TaskLocking, data);
  };

  const handleInputChange = (
    inputType,
    inputValue,
    currentGroupTaskId,
    groupTaskParameterName,
    notes
  ) => {
    const typingMessage = () => {
      sendTypingMessage(
        inputType,
        currentGroupTaskId,
        groupTaskParameterName,
        inputValue,
        notes
      );
    };

    if (taskLockedByUserId !== "") return;

    if (Date.now() >= lastChange.current || lastChange.current === undefined) {
      lastChange.current = Date.now() + 1000;

      typingMessage();
    } else {
      clearTimeout(typingTimer.current);
    }

    typingTimer.current = setTimeout(() => typingMessage(), 1000);
  };

  const getInitialFormValues = () => {
    let initialValues = {};

    groupTaskParameters.forEach((groupTaskParameter) => {
      initialValues[
        `${currentGroupTask.Id}-${step}-${groupTaskParameter.parameterName}`
      ] = getDefaultValue(groupTaskParameter);
    });

    return initialValues;
  };

  return (
    <Form
      form={groupTasksContext.form}
      layout="vertical"
      id="groupTasksViewModalRequiredInputsForm"
      initialValues={getInitialFormValues()}
    >
      {groupTaskParameters.map((groupTaskParameter, index) => {
        switch (groupTaskParameter.type) {
          case "text":
            return (
              <MyFormItem
                key={index}
                t={t}
                labelDisplayName={groupTaskParameter.displayName}
                isGlobal={groupTaskParameter.global}
                name={`${currentGroupTask.Id}-${step}-${groupTaskParameter.parameterName}`}
              >
                <Input
                  id={`${currentGroupTask.Id}-${step}-${groupTaskParameter.parameterName}`}
                  disabled={taskLockedByUserId !== ""}
                  onChange={(e) =>
                    handleInputChange(
                      "text",
                      e.target.value,
                      currentGroupTask.Id,
                      groupTaskParameter.parameterName
                    )
                  }
                />
              </MyFormItem>
            );
          case "number":
            return (
              <MyFormItem
                key={index}
                t={t}
                labelDisplayName={groupTaskParameter.displayName}
                isGlobal={groupTaskParameter.global}
                name={`${currentGroupTask.Id}-${step}-${groupTaskParameter.parameterName}`}
              >
                <InputNumber
                  id={`${currentGroupTask.Id}-${step}-${groupTaskParameter.parameterName}`}
                  style={{ width: "100%" }}
                  disabled={taskLockedByUserId !== ""}
                  max={Number.MAX_SAFE_INTEGER}
                  onChange={(e) =>
                    handleInputChange(
                      "text",
                      e,
                      currentGroupTask.Id,
                      groupTaskParameter.parameterName
                    )
                  }
                />
              </MyFormItem>
            );
          case "textarea":
            return (
              <MyFormItem
                key={index}
                t={t}
                labelDisplayName={groupTaskParameter.displayName}
                isGlobal={groupTaskParameter.global}
                name={`${currentGroupTask.Id}-${step}-${groupTaskParameter.parameterName}`}
              >
                <TextArea
                  id={`${currentGroupTask.Id}-${step}-${groupTaskParameter.parameterName}`}
                  disabled={taskLockedByUserId !== ""}
                  onChange={(e) =>
                    handleInputChange(
                      "text",
                      e.target.value,
                      currentGroupTask.Id,
                      groupTaskParameter.parameterName
                    )
                  }
                />
              </MyFormItem>
            );
          case "select":
            return (
              <MyFormItem
                key={index}
                t={t}
                labelDisplayName={groupTaskParameter.displayName}
                isGlobal={groupTaskParameter.global}
                name={`${currentGroupTask.Id}-${step}-${groupTaskParameter.parameterName}`}
              >
                <SelectComponent
                  id={`${currentGroupTask.Id}-${step}-${groupTaskParameter.parameterName}`}
                  t={t}
                  disabled={taskLockedByUserId !== ""}
                  options={groupTaskParameter.options}
                  parameterName={groupTaskParameter.parameterName}
                  onSelectChange={(value) =>
                    handleInputChange(
                      "select",
                      value,
                      currentGroupTask.Id,
                      groupTaskParameter.parameterName
                    )
                  }
                />
              </MyFormItem>
            );
          case "select_machine":
            return (
              <MyFormItem
                key={index}
                t={t}
                labelDisplayName={groupTaskParameter.displayName}
                isGlobal={groupTaskParameter.global}
              >
                <SelectMachineComponent
                  id={`${currentGroupTask.Id}-${step}-${groupTaskParameter.parameterName}`}
                  disabled={taskLockedByUserId !== ""}
                  t={t}
                  notificationApi={notificationApi}
                  globalInput={groupTaskParameter}
                  onSelectChange={(value, notes) =>
                    handleInputChange(
                      "select",
                      value,
                      currentGroupTask.Id,
                      groupTaskParameter.parameterName,
                      notes
                    )
                  }
                />
              </MyFormItem>
            );
          case "checkbox":
            return (
              <MyFormItem
                key={index}
                t={t}
                style={{
                  marginBottom:
                    groupTaskParameters.length > index &&
                    groupTaskParameters[index + 1].type === "checkbox"
                      ? 0
                      : AppStyle.app.margin,
                }}
                name={`${currentGroupTask.Id}-${step}-${groupTaskParameter.parameterName}`}
              >
                <Checkbox
                  id={`${currentGroupTask.Id}-${step}-${groupTaskParameter.parameterName}`}
                  disabled={taskLockedByUserId !== ""}
                  checked={
                    groupTasksContext.selectInputs[
                      groupTaskParameter.parameterName
                    ]?.value
                  }
                  onChange={(e) => {
                    groupTasksContext.setSelectInputs((prevState) => ({
                      ...prevState,
                      [groupTaskParameter.parameterName]: {
                        value: e.target.checked,
                      },
                    }));

                    handleInputChange(
                      "checkbox",
                      e.target.checked,
                      currentGroupTask.Id,
                      groupTaskParameter.parameterName
                    );
                  }}
                >
                  <MyFormItemLabel
                    t={t}
                    displayName={groupTaskParameter.displayName}
                    isGlobal={groupTaskParameter.global}
                  />
                </Checkbox>
              </MyFormItem>
            );
          case "select":
            return (
              <MyFormItem
                key={index}
                t={t}
                labelDisplayName={groupTaskParameter.displayName}
                isGlobal={groupTaskParameter.global}
                name={`${currentGroupTask.Id}-${step}-${groupTaskParameter.parameterName}`}
              >
                <SelectComponent
                  id={`${currentGroupTask.Id}-${step}-${groupTaskParameter.parameterName}`}
                  t={t}
                  disabled={taskLockedByUserId !== ""}
                  options={groupTaskParameter.options}
                  parameterName={groupTaskParameter.parameterName}
                  onSelectChange={(value) =>
                    handleInputChange(
                      "select",
                      value,
                      currentGroupTask.Id,
                      groupTaskParameter.parameterName
                    )
                  }
                />
              </MyFormItem>
            );
          default:
            return (
              <Typography.Text type="danger" key={index}>
                Type <b>{groupTaskParameter.type}</b> not implemented. Was
                specified in: <b>{groupTaskParameter.displayName}</b> <br />
                <br />
              </Typography.Text>
            );
        }
      })}
    </Form>
  );

  /*return (
    <Form layout="vertical" id="groupTasksViewModalRequiredInputsForm">
      {groupTaskParameters.map((groupTaskParameter, index) => {
        switch (groupTaskParameter.type) {
          case "text":
            return (
              <MyFormItem
                key={index}
                t={t}
                labelDisplayName={groupTaskParameter.displayName}
                isGlobal={groupTaskParameter.global}
              >
                <Input
                  id={`${currentGroupTask.Id}-${step}-${groupTaskParameter.parameterName}`}
                  defaultValue={getDefaultValue(groupTaskParameter)}
                  disabled={taskLockedByUserId !== ""}
                  onChange={(e) =>
                    handleInputChange(
                      "text",
                      e.target.value,
                      currentGroupTask.Id,
                      groupTaskParameter.parameterName
                    )
                  }
                />
              </MyFormItem>
            );
          case "number":
            return (
              <MyFormItem
                key={index}
                t={t}
                labelDisplayName={groupTaskParameter.displayName}
                isGlobal={groupTaskParameter.global}
              >
                <InputNumber
                  id={`${currentGroupTask.Id}-${step}-${groupTaskParameter.parameterName}`}
                  style={{ width: "100%" }}
                  defaultValue={getDefaultValue(groupTaskParameter)}
                  disabled={taskLockedByUserId !== ""}
                  max={Number.MAX_SAFE_INTEGER}
                  onChange={(e) =>
                    handleInputChange(
                      "text",
                      e,
                      currentGroupTask.Id,
                      groupTaskParameter.parameterName
                    )
                  }
                />
              </MyFormItem>
            );
          case "textarea":
            return (
              <MyFormItem
                key={index}
                t={t}
                labelDisplayName={groupTaskParameter.displayName}
                isGlobal={groupTaskParameter.global}
              >
                <TextArea
                  id={`${currentGroupTask.Id}-${step}-${groupTaskParameter.parameterName}`}
                  defaultValue={getDefaultValue(groupTaskParameter)}
                  disabled={taskLockedByUserId !== ""}
                  onChange={(e) =>
                    handleInputChange(
                      "text",
                      e.target.value,
                      currentGroupTask.Id,
                      groupTaskParameter.parameterName
                    )
                  }
                />
              </MyFormItem>
            );
          case "select_machine":
            return (
              <MyFormItem
                key={index}
                t={t}
                labelDisplayName={groupTaskParameter.displayName}
                isGlobal={groupTaskParameter.global}
              >
                <SelectMachineComponent
                  id={`${currentGroupTask.Id}-${step}-${groupTaskParameter.parameterName}`}
                  disabled={taskLockedByUserId !== ""}
                  t={t}
                  notificationApi={notificationApi}
                  globalInput={groupTaskParameter}
                  onSelectChange={(value, notes) =>
                    handleInputChange(
                      "select",
                      value,
                      currentGroupTask.Id,
                      groupTaskParameter.parameterName,
                      notes
                    )
                  }
                />
              </MyFormItem>
            );
          case "checkbox":
            return (
              <MyFormItem
                key={index}
                t={t}
                style={{
                  marginBottom:
                    groupTaskParameters.length > index &&
                    groupTaskParameters[index + 1].type === "checkbox"
                      ? 0
                      : AppStyle.app.margin,
                }}
              >
                <Checkbox
                  id={`${currentGroupTask.Id}-${step}-${groupTaskParameter.parameterName}`}
                  disabled={taskLockedByUserId !== ""}
                  checked={
                    groupTasksContext.selectInputs[
                      groupTaskParameter.parameterName
                    ]?.value
                  }
                  onChange={(e) => {
                    groupTasksContext.setSelectInputs((prevState) => ({
                      ...prevState,
                      [groupTaskParameter.parameterName]: {
                        value: e.target.checked,
                      },
                    }));

                    handleInputChange(
                      "checkbox",
                      e.target.checked,
                      currentGroupTask.Id,
                      groupTaskParameter.parameterName
                    );
                  }}
                >
                  <MyFormItemLabel
                    t={t}
                    displayName={groupTaskParameter.displayName}
                    isGlobal={groupTaskParameter.global}
                  />
                </Checkbox>
              </MyFormItem>
            );
          case "select":
            return (
              <MyFormItem
                key={index}
                t={t}
                labelDisplayName={groupTaskParameter.displayName}
                isGlobal={groupTaskParameter.global}
              >
                <SelectComponent
                  id={`${currentGroupTask.Id}-${step}-${groupTaskParameter.parameterName}`}
                  t={t}
                  disabled={taskLockedByUserId !== ""}
                  options={groupTaskParameter.options}
                  parameterName={groupTaskParameter.parameterName}
                  onSelectChange={(value) =>
                    handleInputChange(
                      "select",
                      value,
                      currentGroupTask.Id,
                      groupTaskParameter.parameterName
                    )
                  }
                />
              </MyFormItem>
            );

          default:
            return (
              <Typography.Text type="danger" key={index}>
                Type <b>{groupTaskParameter.type}</b> not implemented. Was
                specified in: <b>{groupTaskParameter.displayName}</b> <br />
                <br />
              </Typography.Text>
            );
        }
      })}
    </Form>
  ); */
}

function GroupTaskStepLogHandler({ currentGroupTaskId, log, files }) {
  return (
    <span style={{ whiteSpace: "pre-line" }}>
      {log}
      {files !== "" && files !== " " && (
        <MyAttachments
          attachments={JSON.parse(files)}
          downloadUrl={`${Constants.STATIC_CONTENT_ADDRESS}grouptasks/${currentGroupTaskId}/`}
        />
      )}
    </span>
  );
}
